import csv
import re
import os
import numpy as np
import webcolors
from scipy import spatial


# pt = [221, 182, 137, 255]  # = "burlywood","#DEB887"]
# pt = [0, 102, 109, 255]  # = "burlywood","#DEB887"]


def find_hex_color(hex_color):
    hex_color = hex_color.rstrip('\n')

    if hex_color == "#000000":
        return "Unify_N700"
    elif hex_color == "#8000":
        return "Unify_P500"
    elif hex_color == "#000000":
        return "Unify_N700"
    elif hex_color == "#00000000":
        return hex_color
    elif re.match("#(.*)000000", hex_color):
        if len(hex_color) == 9:
            hex_color = hex_color[:3] + "31353B"
        else:
            hex_color = hex_color[:2] + "31353B"
    hex_color = hex_color.replace(" ","")
    HexNameDict = {}
    script_dir = os.path.dirname(__file__)  # Script directory
    color_mapper_file = os.path.join(script_dir, 'dataset/rgb_dataset.csv')
    with open(color_mapper_file) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        rgb = []
        rgba = []
        for row in csv_reader:
            rgb.append([int(row[2]), int(row[3]), int(row[4])])
            rgba.append([int(row[2]), int(row[3]), int(row[4]), int(row[5])])
            HexNameDict[row[1]] = row[0]
        RGB = np.array(rgb)
    try:
        hex_color_rgb = webcolors.hex_to_rgb(hex_color)
        alpha = 255
    except ValueError:
        new_hex = "#" + format(hex_color[3:])
        alpha_hex = hex_color[1:3]
        hex_color_rgb = webcolors.hex_to_rgb(new_hex)
        alpha = int(alpha_hex, 16)

    pt = [hex_color_rgb.red, hex_color_rgb.green, hex_color_rgb.blue]
    # Lookup color name using Hex:ColorName dictionary:
    nearest = spatial.KDTree(RGB).query(pt)
    NearestRGB = (RGB[nearest[1]])

    if alpha != 255:
        # special case with alpha
        # find nearest alpha
        pt = [hex_color_rgb.red, hex_color_rgb.green, hex_color_rgb.blue, alpha]
        colors = []
        for row in rgba:
            if row[0] == pt[0] and row[1] == pt[1] and row[2] == pt[2]:
                colors.append(row)
        if len(colors) > 1:
            RGBA = np.array(colors)
            nearest_alpha = (RGBA[spatial.KDTree(RGBA).query(pt)[1]])
            if nearest_alpha[3] != 255:
                s = '#' \
                    + format(nearest_alpha[3], 'x').zfill(2) \
                    + format(nearest_alpha[0], 'x').zfill(2) \
                    + format(nearest_alpha[1], 'x').zfill(2) \
                    + format(nearest_alpha[2], 'x').zfill(2)
            else:
                s = '#' \
                    + format(nearest_alpha[0], 'x').zfill(2) \
                    + format(nearest_alpha[1], 'x').zfill(2) \
                    + format(nearest_alpha[2], 'x').zfill(2)
            ColorHex = s.upper()
            ColorDiff = \
                '(' + '{0:+d}'.format(nearest_alpha[3] - pt[3]) \
                + ',' + '{0:+d}'.format(nearest_alpha[0] - pt[0]) \
                + ',' + '{0:+d}'.format(nearest_alpha[1] - pt[1]) \
                + ',' + '{0:+d}'.format(nearest_alpha[2] - pt[2]) \
                + ')'
        else:
            s = '#' \
                + format(NearestRGB[0], 'x').zfill(2) \
                + format(NearestRGB[1], 'x').zfill(2) \
                + format(NearestRGB[2], 'x').zfill(2)
            ColorHex = s.upper()
            ColorDiff = \
                '(' + '{0:+d}'.format(NearestRGB[0] - pt[0]) \
                + ',' + '{0:+d}'.format(NearestRGB[1] - pt[1]) \
                + ',' + '{0:+d}'.format(NearestRGB[2] - pt[2]) \
                + ')'
            nearest_alpha = NearestRGB
        try:
            ColorName = HexNameDict[ColorHex]
        except:
            ColorName = ""

        # print('Nearest color name to input RGB '
        #       + str(pt)
        #       + '(' + hex_color + ')'
        #       + ' is "' + ColorName + '"'
        #       + ' ' + ColorHex
        #       + ' ' + str(nearest_alpha)
        #       + ', ' + ColorDiff
        #       + '.')
        return ColorName

    else:
        s = '#' \
            + format(NearestRGB[0], 'x').zfill(2) \
            + format(NearestRGB[1], 'x').zfill(2) \
            + format(NearestRGB[2], 'x').zfill(2)
        ColorHex = s.upper()
        ColorDiff = \
            '(' + '{0:+d}'.format(NearestRGB[0] - pt[0]) \
            + ',' + '{0:+d}'.format(NearestRGB[1] - pt[1]) \
            + ',' + '{0:+d}'.format(NearestRGB[2] - pt[2]) \
            + ')'
        try:
            ColorName = HexNameDict[ColorHex]
        except:
            ColorName = ""

        print('Nearest color name to input RGB '
              + str(pt)
              + '(' + hex_color + ')'
              + ' is "' + ColorName + '"'
              + ' ' + ColorHex
              + ' ' + str(NearestRGB)
              + ', ' + ColorDiff
              + '.')
        return ColorName

# print(find_hex_color("#AF3E14"))
find_hex_color("#D6DFEB")


import csv
import glob
import os
import re

from unify_package.utils import is_test_file, is_hidden_file, is_build_directory, is_strings_values, \
    ignoredFiles, is_manifest, is_gradle_contains_unify_principle

color_mapper = {}
file = [
    "/Users/nakama/android-tokopedia-core/tkpdseller/src/main/java/com/tokopedia/seller/base/view/activity/BaseToolbarActivity.java"]
re_color_android_compat = 'Color\.WHITE|Color\.BLACK|Color\.LTGRAY|Color\.GRAY|Color\.TRANSPARENT|Color\.BLUE|Color\.RED|Color\.GREEN'
script_dir = os.path.dirname(__file__)  # Script directory
custom_color_mapper_path = os.path.join(script_dir, 'dataset/neutral_to_unify.csv')
current_file = "test"


def find_color_in_line(file_to_refactor):
    file_changes = 0
    p = re.compile(
        'com\.[\w_]+\.[\w_]+\.R\.color\.[\w_]+|R\.color\.\w+|\.color\.\w+|@color/\w+|android.R.color.\w+|@android:color/\w+|Color\.WHITE|Color\.BLACK|Color\.LTGRAY|Color\.GRAY|Color\.TRANSPARENT|Color\.BLUE|Color\.RED|Color\.GREEN')
    for file in file_to_refactor:
        if not is_file_ignored(file):
            global current_file
            current_file = file

            with open(file) as f:
                data = f.read()
                target_data = data
                new_data = p.sub(replace_with_color_mapper, target_data)
                is_new_data = 1 if new_data != data else 0
            f.close()

            if is_new_data == 1:
                file_changes += 1
                with open(file, 'wt') as f:
                    f.write(new_data)
                f.close()
    return file_changes


def replace_with_color_mapper(m):
    global current_file
    replace = m.group(0)
    color_except_full_path = replace
    should_append_full_path = False

    androidColorCompat = re.search(re_color_android_compat, replace)

    if str(replace).startswith("com.") or str(replace).startswith("android."):
        # color_except_full_path -> from com.tokopedia.*.R.color.Unify_N700 to R.color.Unify_N700
        color_except_full_path = re.findall('R.color.\w+', replace)[0]
        should_append_full_path = True
    elif str(replace).startswith("R.color."):
        should_append_full_path = False

    if should_append_full_path:
        try:
            replace = color_mapper[color_except_full_path]
            replace = "com.tokopedia.unifyprinciples." + replace
        except Exception as e:
            e
    elif androidColorCompat:
        # Color.WHITE/Color.BLACK/etcetc
        try:
            if current_file.endswith('Activity.java') or current_file.endswith('Activity.kt'):
                replace = 'androidx.core.content.ContextCompat.getColor(this,' + 'com.tokopedia.unifyprinciples.' + \
                          color_mapper[m.group(0)] + ')'
            else:
                replace = 'androidx.core.content.ContextCompat.getColor(context,' + 'com.tokopedia.unifyprinciples.' + \
                          color_mapper[m.group(0)] + ')'
        except Exception as e:
            pass
    else:
        try:
            replace = color_mapper[m.group(0)]
        except:
            pass

    return replace


def get_color_map():
    colors = {}
    with open(custom_color_mapper_path) as mapping_classes:
        csv_reader = csv.reader(mapping_classes)
        for idx, class_map in enumerate(csv_reader):
            old_color = class_map[0]
            new_color = class_map[1]
            colors[old_color] = new_color
    return colors


def is_dir_ignored(file_path):
    if (
            is_test_file(file_path) or
            is_hidden_file(file_path) or
            is_build_directory(file_path) or
            is_strings_values(file_path) or
            is_manifest(file_path)
    ):
        return True
    for ignored_file in ignoredFiles:
        if file_path.endswith(ignored_file):
            return True
    return False


def get_file_path(project_path, file):
    return project_path + "/" + file


def is_file_ignored(file_path):
    return is_dir_ignored(file_path)


def traverse_folders(project_path):
    files = glob.glob(project_path + "/src/**/*.xml", recursive=True) + glob.glob(project_path + "/src/**/*.kt", recursive=True) + glob.glob(
        project_path + "/src/**/*.java", recursive=True)
    return files


def replace_neutral_with_unify(path):
    localCount = 0
    localCount += is_gradle_contains_unify_principle(path)
    global color_mapper
    color_mapper = get_color_map()
    localCount += find_color_in_line(traverse_folders(path))
    if localCount > 0:
        is_gradle_contains_unify_principle(path)
    return localCount



import csv
import glob
import os
import re

from unify_package.utils import is_test_file, is_hidden_file, is_build_directory, is_strings_values, \
    ignoredFiles, is_manifest

color_mapper = {}
script_dir = os.path.dirname(__file__)  # Script directory
custom_color_mapper_path = os.path.join(script_dir, 'dataset/FinalCustomColor.csv')


def find_color_in_line(file_to_refactor):
    file_changes = 0
    p = re.compile(
        'com\.[\w_]+\.[\w_]+\.R\.color\.[\w_]+|R\.color\.\w+|@color/\w+|android.R.color.\w+|@android:color/\w+')
    for file in file_to_refactor:
        if not is_file_ignored(file):
            with open(file) as f:
                data = f.read()
                target_data = data
                new_data = p.sub(replace_with_color_mapper, target_data)
                is_new_data = 1 if new_data != data else 0
            f.close()

            if is_new_data == 1:
                file_changes += 1
                with open(file, 'wt') as f:
                    f.write(new_data)
                f.close()
    return file_changes


def regex_find_color(color):
    if color.startswith("@color/"):
        return color
    else:
        return r'com.*' + color + '\b|' + color + '\b'


def replace_with_color_mapper(m):
    replace = m.group(0)
    color_except_full_path = replace
    should_append_full_path = False

    if str(replace).startswith("com.") or str(replace).startswith("android."):
        color_except_full_path = re.findall('R.color.\w+', replace)[0]
        should_append_full_path = True
    elif str(replace).startswith("R.color."):
        should_append_full_path = True

    if should_append_full_path:
        try:
            replace = color_mapper[color_except_full_path]
            replace = "com.tokopedia.unifyprinciples." + replace
        except Exception as e:
            e
    else:
        try:
            replace = color_mapper[m.group(0)]
        except:
            pass

    return replace


def get_color_map():
    colors = {}
    with open(custom_color_mapper_path) as mapping_classes:
        csv_reader = csv.reader(mapping_classes)
        for idx, class_map in enumerate(csv_reader):
            old_color = class_map[0]
            new_color = class_map[1]
            colors[old_color] = new_color
    return colors


def is_dir_ignored(file_path):
    if (
            is_test_file(file_path) or
            is_hidden_file(file_path) or
            is_build_directory(file_path) or
            is_strings_values(file_path) or
            is_manifest(file_path)
    ):
        return True
    for ignored_file in ignoredFiles:
        if file_path.endswith(ignored_file):
            return True
    return False


def get_file_path(project_path, file):
    return project_path + "/" + file


def is_file_ignored(file_path):
    return is_dir_ignored(file_path)


def traverse_folders(project_path):
    files = glob.glob(project_path + "/src/**/*.xml", recursive=True) + glob.glob(project_path + "/src/**/*.kt", recursive=True) + glob.glob(
        project_path + "/src/**/*.java", recursive=True)
    return files


def is_gradle_contains_unify_principle(rootPath):
    try:
        unify_principle_dependencies = "implementation rootProject.ext.unifyDependencies.principles"
        gradle_path = rootPath + os.sep + "build.gradle"
        is_appended = False
        with open(gradle_path) as file:
            is_need_to_append = unify_principle_dependencies not in file.read()

        data = ""
        with open(gradle_path) as file:
            for f in file:
                no_space = f.replace(" ", "")
                if no_space.startswith("implementation") and not is_appended and is_need_to_append:
                    data = data + "\t" + unify_principle_dependencies + os.linesep
                    is_appended = True
                data = data + f
        file.close()

        if is_appended:
            with open(gradle_path, 'wt') as file:
                file.write(data)
            file.close()
    except FileNotFoundError:
        return


def replace_custom_color_with_unify(path):
    global color_mapper
    color_mapper = get_color_map()
    return find_color_in_line(traverse_folders(path))



import csv
import fileinput
import os
import re
import sys
from pathlib import Path

ignoredFiles = [
    ".iml",
    ".pro",
    ".jar",
    ".bat",
    ".js",
    ".yml",
    ".txt",
    ".so",
    ".bundle",
    ".meta",
    ".properties",
    ".json",
    ".apk",
    ".png",
    ".jpg",
    ".jpeg",
    ".svg",
    ".aar",
    ".class",
    ".textClipping",
    ".keystore",
    ".lock",
    ".ttf",
    ".py",
    ".hprof",
    "yarn",
    "/raw",
    "BUCK",
    "gradlew",
    "buck-out",
    "CODEOWNERS",
    "aars",
    "buckhelper",
    "buckw",
    "build.gradle",
    "gradle",
    ".webp",
    "dimens.xml",
    "styles.xml"
]

path = Path(os.getcwd())
color_mapper_file = path.parent.__str__() + "/color+moveresource/dataset/neutral_to_unify.csv"
color_custom_mapper_file = path.parent.__str__() + "/dataset/color_util_dataset/FinalCustomColor.csv"


def is_colors_res(file_path):
    return file_path.endswith("colors.xml")


def is_hidden_file(relative_path):
    file_name = os.path.basename(relative_path)
    return file_name.startswith(".")


def is_test_file(file_path):
    return file_path.endswith("androidTest")


def is_build_directory(relative_path):
    return relative_path.find("/androidTest/") != -1 or relative_path.find("/test/") != -1


def is_model_directory(relative_path):
    return relative_path.find("/model/") != -1 or relative_path.find("/models/") != -1


def is_drawable_file(relative_path):
    return re.search(r"/res/drawable", relative_path)


def is_colors_values(relative_path):
    return re.search(r"/res/values.+colors", relative_path)
    pass


def is_strings_values(relative_path):
    return re.search(r"/res/values.+strings", relative_path)
    pass


def is_manifest(relative_path):
    return re.search(r"AndroidManifest.xml", relative_path)
    pass


def is_vector_file(relative_path):
    with open(relative_path) as file:
        is_need_to_append = '</vector>' in file.read()
    return is_need_to_append


def is_dir_ignored(project_path, file_path):
    relative_path = file_path.replace(project_path, "")
    if (
            is_test_file(file_path) or
            is_hidden_file(relative_path) or
            is_build_directory(relative_path) or
            is_strings_values(relative_path)
    ):
        return True
    for ignored_file in ignoredFiles:
        if relative_path.endswith(ignored_file):
            return True
    return False


def get_file_path(project_path, file):
    return project_path + "/" + file


def is_file_ignored(project_path, file_path):
    return is_dir_ignored(project_path, file_path)


def generateCompleteColorMap(source_csv_file, new_csv_file):
    print("asd")
    with open(new_csv_file, 'w',
              newline='') as myfile:
        for line in open(source_csv_file):
            print(line)
            split_line = line.split(',')
            color_key = 'R.color.' + split_line[0]
            color_key_value = 'R.color.' + split_line[1].strip()
            color_key_xml = '@color/' + split_line[0]
            color_key_xml_value = '@color/' + split_line[1].strip()

            writer = csv.writer(myfile)
            writer.writerow([color_key, color_key_value])
            writer.writerow([color_key_xml, color_key_xml_value])


def is_gradle_contains_unify_principle(rootPath):
    try:
        count = 0
        unify_principle_dependencies = "implementation rootProject.ext.unifyDependencies.principles"
        gradle_path = rootPath + os.sep + "build.gradle"
        is_appended = False
        with open(gradle_path) as file:
            is_need_to_append = unify_principle_dependencies not in file.read()

        data = ""
        with open(gradle_path) as file:
            for f in file:
                no_space = f.replace(" ", "")
                if no_space.startswith("implementation") and not is_appended and is_need_to_append:
                    data = data + "\t" + unify_principle_dependencies + os.linesep
                    is_appended = True
                data = data + f
        file.close()

        if is_appended:
            count += 1
            with open(gradle_path, 'wt') as file:
                file.write(data)
            file.close()
        return count
    except FileNotFoundError:
        return

# generateCompleteColorMap()



import os
import sys
from pathlib import Path

from find_gradle_file import get_mapper_all_colors
from utility.change_style_theme_with_unify_day_night import change_style_theme_with_unify_day_night
from utility.get_child_module import get_child_module
from utility.replace_color_with_all_dependency import replace_color_with_all_dependency
from utility.replace_hardcode_color_with_unify import replace_hardcode_color_with_unify
from utility.replace_neutral_with_unify import replace_color_with_neutral
from utility.replace_custom_color_with_unify import replace_custom_color_with_unify


def main(root_path, module_path):
    get_mapper_all_colors(root_path)
    change_style_theme_with_unify_day_night(root_path + os.sep + module_path)
    replace_color_with_all_dependency(root_path, module_path)


def process_arguments():
    root_path = ""
    module_path = ""

    try:
        root_path = sys.argv[1]
    except:
        print("Please provide android tokopedia root path")
        pass

    try:
        module_path = sys.argv[2]
    except:
        print("Please provide module absolute path")

    if root_path != "" and module_path != "":
        main(root_path, module_path)
    else:
        print("Failed...")


if __name__ == '__main__':
    process_arguments()



import csv
import fileinput
import os
import re
import sys

from unify_package.find_nearest_hex_color import find_hex_color

ignoredFiles = ["pre_release_inspector"]
project_location = "/Users/nakama/android-tokopedia-core"
unify_principle_dependencies = "implementation rootProject.ext.unifyDependencies.principles"
color_mapper_file = os.path.dirname(__file__) + "/dataset/color-mapper.csv"

# class name starts with _ is a utility class
YOUR_DATA_MAPPER_STEP2_PATH = os.path.dirname(__file__) + "/dataset/"
raw_custom_color = YOUR_DATA_MAPPER_STEP2_PATH + "_RawCustomColor.txt"
unreference_custom_color = YOUR_DATA_MAPPER_STEP2_PATH + "_UnreferenceCustomColor.txt"
unique_custom_color = YOUR_DATA_MAPPER_STEP2_PATH + "_UniqueCustomColor.txt"
final_custom_color = YOUR_DATA_MAPPER_STEP2_PATH + "_FinalCustomColor.txt"
raw_colors_gradle_path = YOUR_DATA_MAPPER_STEP2_PATH + "_RawColorsGradlePath.txt"
# ================================================================================================================#

list_of_path_custom_colors = []

totalColorFound = 0
totalColorNotFound = 0
totalValueColorNotFound = 0
totalModuleScan = 0
totalGradleNotContainsPrinciple = 0


def is_root_folder(filePath):
    root_file = None
    if any(File.endswith("build.gradle") for File in os.listdir(filePath)):
        root_file = filePath
    return root_file


def get_file_path(project_location, file):
    return project_location + "/" + file


def traverse_color_res(project_location, current_location):
    global totalModuleScan
    files = os.listdir(current_location)
    files.sort()
    for file in files:
        file_path = get_file_path(current_location, file)

        if os.path.isdir(file_path):
            root_path = is_root_folder(file_path)
            if root_path:
                totalModuleScan += 1
                # find_string_color(root_path)
                values_folder_found(root_path)
            else:
                traverse_color_res(project_location, file_path)


def find_string_color(root_path):
    global totalColorFound
    global totalColorNotFound
    global totalValueColorNotFound
    try:
        values_path = root_path + os.sep + 'src/main/res/values'
        colors_path_re = "strings.xml"

        if any(re.match(colors_path_re, File) for File in os.listdir(values_path)):
            for file_name in os.listdir(values_path):
                if re.match(colors_path_re, file_name):
                    path = values_path + "/" + file_name
                    new_color_target = []
                    new_file = []
                    with open(path, 'r+') as f:

                        for line in f:
                            no_space = line.replace(" ", "")
                            if no_space.startswith('<color'):
                                new_color_target.append(line)
                                totalColorFound += 1
                            else:
                                new_file.append(line)

                        f.seek(0)
                        for line in new_file:
                            f.write(line)
                        f.truncate()
                    f.close()

                    migrate_string_color_to_color_file(values_path, new_color_target)

                    list_of_path_custom_colors.append(file_name + os.sep + values_path)
        else:
            totalColorNotFound += 1
    except FileNotFoundError:
        totalValueColorNotFound += 1


def migrate_string_color_to_color_file(root_path, values):
    if len(values) == 0:
        return
    colors_path = root_path + "/" + "colors.xml"
    data = ""
    is_appended = False
    with open(colors_path) as file:
        for f in file:
            no_space = f.replace(" ", "")
            if no_space.startswith("<color") and not is_appended:
                for line in values:
                    data = data + line
                is_appended = True
                data = data + f
            else:
                data = data + f
    file.close()
    if data == "":
        data = data + '<?xml version="1.0" encoding="utf-8"?>'
        data = data + "<resources>"
        for line in values:
            data = data + line
        data = data + "</resources>"
    with open(colors_path, 'wt') as file:
        file.write(data)
    file.close()


def values_folder_found(rootPath):
    global totalColorFound
    global totalColorNotFound
    global totalValueColorNotFound
    if os.path.basename(os.path.normpath(rootPath)) in ignoredFiles:
        return
    try:
        colors_path = rootPath + os.sep + 'src/main/res/values'
        colors_path_re = "(.*?color.*?).xml"

        if any(re.match(colors_path_re, File) for File in os.listdir(colors_path)):
            is_gradle_contains_unify_principle(rootPath)
            for colorsFile in os.listdir(colors_path):
                if re.match(colors_path_re, colorsFile):
                    totalColorFound += 1
                    list_of_path_custom_colors.append(colors_path + os.sep + colorsFile)
        else:
            totalColorNotFound += 1
    except FileNotFoundError:
        totalValueColorNotFound += 1


def is_colors_xml_exist(colors_path_re, colors_path):
    any(re.match(colors_path_re, File) for File in os.listdir(colors_path))


def get_all_custom_color():
    with open(raw_custom_color, "w") as combineFile:
        for filePath in list_of_path_custom_colors:
            with open(filePath) as f:
                for line in f:
                    no_space = line.replace(" ", "")
                    if no_space.startswith('<color'):
                        color_name = re.findall('name="(.*)"', line.lstrip())
                        color_hex = re.findall('>(.*)</', line.lstrip())[0]

                        if color_hex.startswith("@android:color/"):
                            if color_hex == "@android:color/black":
                                change_color = "#000000"
                            elif color_hex == "@android:color/white":
                                change_color = "#ffffff"
                            elif color_hex == "@android:color/transparent":
                                change_color = "#00000000"

                            combineFile.write(color_name[0] + ',' + change_color + '\n')
                        else:
                            combineFile.write(color_name[0] + ',' + color_hex + '\n')
            f.close()
    combineFile.close()


def remove_transparent_and_dark_mode_support():
    for line in fileinput.input(raw_custom_color, inplace=1):
        hex_value = line.split(',')[1]
        color_name = line.split(',')[0]

        #If color in module is dark mode support custom color, remove from mapper
        if "dark_mode_support" in color_name:
            continue

        if "#00000000" not in hex_value:
            sys.stdout.write(line)


""
'Sometimes the color is reference to other colors eg: <color name=first_green>@color/medium_green</color>'
'Here we need to change the reference color into hex color like <color name=first_green>#42b549</color>'
""
def change_hardcoded_reference_color():
    uniqueColorFile = open(unreference_custom_color, 'w')
    for line in open(unique_custom_color, 'r'):
        line_split = line.split(',')
        color_hex = line_split[1]

        # color_red_main,@color/red_primary
        color_contains_another_reference = color_hex.startswith('@color')

        if color_contains_another_reference:
            hex_by_color_reference = find_reference_color(color_hex)
            if hex_by_color_reference:
                uniqueColorFile.write(line.replace(color_hex, hex_by_color_reference))
            else:
                # maybe the color is already unify reference
                color_unify = find_reference_in_unify_principle(color_hex)
                if color_unify:
                    uniqueColorFile.write(line.replace(color_hex, color_unify))
                else:
                    uniqueColorFile.write(line)
        else:
            uniqueColorFile.write(line)


def generate_color_resource():
    # _RawCustomColor.txt
    get_all_custom_color()
    remove_transparent_and_dark_mode_support()

    # _RawColorsGradlePath
    create_color_path()

    # _UniqueCustomColor
    remove_duplicate()

    # _UnreferenceCustomColor
    change_hardcoded_reference_color()

    # _FinalCustomColor
    change_value_hex_to_unify()
    remove_null_value()
    adjustUnifyAttribute()

    # FinalCustomColor
    convertFinalColorToCsv()


def remove_null_value():
    for line in fileinput.input(final_custom_color, inplace=1):
        if "null" not in line:
            sys.stdout.write(line)


def convertFinalColorToCsv():
    with open(final_custom_color, 'r') as infile, open(
            YOUR_DATA_MAPPER_STEP2_PATH + "FinalCustomColor.csv", 'w') as outfile:
        stripped = (line.strip() for line in infile)
        lines = (line.split(",") for line in stripped if line)
        writer = csv.writer(outfile)
        writer.writerows(lines)


# Change to R.color.white, Unify_n0
# Into R.color.white, R.color.Unify_n0
def adjustUnifyAttribute():
    for line in fileinput.input(final_custom_color, inplace=1):
        split_line = line.split(',')
        color_name = split_line[0]
        color_unify = split_line[1]

        if "@color/" not in color_unify:
            sys.stdout.write('R.color.' + color_name + ',R.color.' + color_unify)
            sys.stdout.write('@color/' + color_name + ',@color/' + color_unify)
        else:
            sys.stdout.write('R.color.' + color_name + ',R.color.' + color_unify.split("/")[1])
            sys.stdout.write('@color/' + color_name + ',' + color_unify)

    fileinput.close()


def create_color_path():
    with open(raw_colors_gradle_path, "w") as colorsPath:
        for idx, line in enumerate(list_of_path_custom_colors):
            if idx != 0 & idx != len(list_of_path_custom_colors):
                colorsPath.write("\n")
            colorsPath.write(line)
    colorsPath.close()


def remove_duplicate():
    lines_seen = set()  # holds lines already seen
    outfile = open(unique_custom_color, "w")
    for line in open(raw_custom_color, "r"):
        if line not in lines_seen:  # not a duplicate
            outfile.write(line)
            lines_seen.add(line)
    outfile.close()


def change_value_hex_to_unify():
    finalFile = open(final_custom_color, "w")
    for line in open(unreference_custom_color, "r"):
        line_split = line.split(',')
        try:
            finalFile.write(line_split[0] + ',' + find_hex_color(line_split[1]) + '\n')
        except:
            finalFile.write(line)


def find_reference_in_unify_principle(color_hex):
    color_hex = color_hex.rstrip("\n")
    with open(color_mapper_file) as mapping_classes:
        csv_reader = csv.reader(mapping_classes)
        for class_map in csv_reader:
            if color_hex == class_map[0]:
                return class_map[1]
    mapping_classes.close()


def find_reference_color(color_match):
    with open(unique_custom_color, 'r') as f:
        lines = f.readlines()
        color_match = color_match.rstrip('\n')
        for line in lines:
            color_name = line.split(',')[0]
            color_without_attribute = color_match.replace('@color/', '')

            if color_without_attribute == color_name:
                color_hex = line.split(',')[1]
                return color_hex


def is_gradle_contains_unify_principle(rootPath):
    global totalGradleNotContainsPrinciple
    gradle_path = rootPath + os.sep + "build.gradle"
    with open(gradle_path, "r") as f:
        if unify_principle_dependencies in f.read():
            totalGradleNotContainsPrinciple += 0
        else:
            totalGradleNotContainsPrinciple += 1
    f.close()


def get_unify_resource():
    for line in fileinput.input(color_mapper_file, inplace=1):
        color_name = re.findall('name="(.*)"', line.lstrip())
        color_hex = re.findall('>(.*)</', line.lstrip())
        sys.stdout.write(color_name[0] + ',' + color_hex[0] + '\n')


def get_mapper_all_colors(root_tokped_path):
    traverse_color_res(root_tokped_path, root_tokped_path)
    generate_color_resource()




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='unify_script',
    version='0.1.4.8',
    author='Lukas Kristianto | Yehezkiel',
    author_email='lukas.kristianto@tokopedia.com',
    description='Unify color script automation converter',
    packages=['unify_package'],
    install_requires=['numpy','webcolors','scipy', 'pathlib'],
    script=['unify_package/unify_script'],
    include_package_data=True,
    classifier=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import json
import os
import re

exclude_module = [
    ':features:content:play',
    ':features:content:play_widget',
    ':features:content:play_common',
    ':features:content:play_broadcaster',
    ':features:discovery:productcard',
    ':features:discovery:carousel_productcard'
]

def get_child_module(project_path, module_path):
    dependency_graph = DependencyGraph(project_path)
    return dependency_graph.traverseAllChildDependencies(module_path)


class DependencyGraph:

    def __init__(self, project_path):
        self.results = {}
        self.project_path = project_path
        includes = self.findIncludedModules(project_path)
        self.ext = self.findProjectExt(project_path)
        for i in includes:
            slash = i.replace(':', '/')
            # for features/blabla/blabla
            normed = slash[1:] if os.path.isabs(slash) else slash
            full_path = os.path.join(project_path, normed)
            dep = self.findProjectDependency(full_path)
            dep_path = [self.ext[x] for x in dep]
            key = ":" + normed.replace("/", ":")
            self.results[key] = dep_path

    def findProjectDependency(self, path):
        """
        Return list of object representing the path and name of project dependency that a module implements
        Input path is the path of module we want to find out
        """
        build_path = path + "/build.gradle"
        d = []
        if os.path.exists(build_path):
            with open(build_path) as f:
                contents = f.read()
                pattern = re.compile(
                    r"[Ii]mplementation \(?project\(rootProject\.ext\.(\w+)\.(\w+)\)\)?")
                result = re.finditer(pattern, contents)
                for r in result:
                    d.append(r.group(1) + "/" + r.group(2))
        return d

    def findProjectExt(self, project_path):
        """
        """
        pattern = r"^\s+(\w+)\s*:\s*\"([\w:-]+)\""
        feature_path = project_path + "/buildconfig/dependencies/dependency-features.gradle"
        library_path = project_path + "/buildconfig/dependencies/dependency-libraries.gradle"
        d = {}
        with open(feature_path) as f:
            contents = f.read()
            results = re.finditer(pattern, contents, re.MULTILINE)
            for result in results:
                d["features/" + result.group(1)] = result.group(2)
        with open(library_path) as f:
            contents = f.read()
            results = re.finditer(pattern, contents, re.MULTILINE)
            for result in results:
                d["libraries/" + result.group(1)] = result.group(2)
        return d

    def findIncludedModules(self, path):
        pathList = [
            path + "/buildconfig/appcompile/compile-customerapp.gradle",
            path + "/buildconfig/appcompile/compile-sellerapp.gradle",
            path + "/buildconfig/appcompile/compile-libraries.gradle"
        ]
        pattern = r'^include [\'\"]([\w:\-_]+)[\'\"]'
        d = set()

        for p in pathList:
            with open(p) as f:
                contents = f.read()
                results = re.finditer(pattern, contents, re.MULTILINE)
                for result in results:
                    d.add(result.group(1))
        return list(d)

    def constructDependencyGraph(self):
        print(json.dumps(self.results, indent=4))

    def traverse(self, path, visited=[]):
        visited += [path]
        for neighbor in self.results[path]:
            if neighbor not in visited:
                visited = self.traverse(neighbor, visited)
        return visited

    def isCommented(self, line):
        comment_rgx = re.compile(r'\s*//')
        if re.search(comment_rgx, line):
            return True
        else:
            return False

    def traverseAllChildDependencies(self, module_path):
        module_key = ":" + module_path.replace("/", ":")
        result = self.traverse(module_key)
        return [x for x in result if x not in exclude_module]


# print(get_child_module('/Users/nakama/IdeaProjects/android-tokopedia-core', 'features/discovery/home'))


ignoreModule = [
    "akamai_bot_lib",
    "user_identification_common",
    "blockcanary",
    "weaver",
    "testapp",
    "instrumentation_test",
    "graphql",
    "fingerprint",
    "dynamic_features",
    'kelontongapp',
    'dynamic-features',
    'projectFilesBackup3',
    'projectFilesBackup4',
    'dynamic-features-sellerapp',
    "buildconfig",
    "applink",
    "iris",
    "linker",
    "lint",
    "smart_recycler_diffutil_helper",
    "tools",
    "circular_view_pager",
    "common_network",
    "device_info",
    "inappupdate",
    "bottom_sheet_builder",
    "locationmanager",
    "logger",
    "permissionchecker",
    "remote_config",
    "remotesourcerequest",
    "track",
    "tracking_optimizer",
    "annotation_proc_central",
    "attachcommon",
    "common_payment",
    "encryption",
    "product_manage_common",
    "profilecommon",
    "screen_recorder",
    "seller_active_common",
    "tokopatch",
    "translator",
    "utils",
    "websocket",
    "youtube_common",
    "tkpdcore",
    "tkpddesign",
    "tkpdabstraction",
    ':features:content:play',
    ':features:content:play_widget',
    ':features:content:play_common',
    ':features:content:play_broadcaster',
    ':features:discovery:productcard',
    ':features:discovery:carousel_productcard'
]


from unify_package.find_gradle_file import get_mapper_all_colors
from unify_package.ignore_module import ignoreModule
from unify_package.get_child_module import get_child_module
from unify_package.replace_custom_color_with_unify import replace_custom_color_with_unify
from unify_package.replace_hardcode_color_with_unify import replace_hardcode_color_with_unify
from unify_package.replace_neutral_to_unify_new import replace_neutral_with_unify


def replace_color(project_path, module_path):
    get_mapper_all_colors(project_path)
    result = get_child_module(project_path, module_path)
    filtered_result = (s for s in result if not any(e in s for e in ignoreModule))
    count = 0
    for r in filtered_result:
        p = project_path + r.replace(":", "/")
        count += replace_neutral_with_unify(p)
        count += replace_custom_color_with_unify(p)
        count += replace_hardcode_color_with_unify(p)
    print(module_path + ": " + str(count) + " file changes")


if __name__ == "__main__":
    # replace_color(sys.argv[1], sys.argv[2])
    replace_color('/Users/nakama/IdeaProjects/android-tokopedia-core', 'features/discovery/home_wishlist')